﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;
using AionCxxLibrary.Utilities;

namespace AionCxxLibrary.Enums
{
    public enum CharacterClass : byte
    {
        [XmlEnum("warrior")]
        Warrior = 0,
        WARRIOR = Warrior,
        [XmlEnum("fighter")]
        Gladiator = 1,
        FIGHTER = Gladiator,
        [XmlEnum("knight")]
        Templar = 2,
        KNIGHT = Templar,
        [XmlEnum("scout")]
        Scout = 3,
        SCOUT = Scout,
        [XmlEnum("assassin")]
        Assassin = 4,
        ASSASSIN = Assassin,
        [XmlEnum("ranger")]
        Ranger = 5,
        RANGER = Ranger,
        [XmlEnum("mage")]
        Mage = 6,
        MAGE = Mage,
        [XmlEnum("wizard")]
        Sorcerer = 7,
        WIZARD = Sorcerer,
        [XmlEnum("elementalist")]
        Spiritmaster = 8,
        ELEMENTALIST = Spiritmaster,
        ELEMENTALLIST = Spiritmaster,
        [XmlEnum("cleric")]
        Priest = 9,
        CLERIC = Priest,
        [XmlEnum("priest")]
        Cleric = 10,
        PRIEST = Cleric,
        [XmlEnum("chanter")]
        Chanter = 11,
        CHANTER = Chanter,
        None = 0xFF,
        ALL = None,
    }

    public class CharacterClassEnum
    {
        public static CharacterClass ForAlias(string alias)
        {
            switch (alias)
            {
                case "warrior": return CharacterClass.Warrior;
                case "fighter": return CharacterClass.Gladiator;
                case "knight": return CharacterClass.Templar;
                case "assassin": return CharacterClass.Assassin;
                case "scout": return CharacterClass.Scout;
                case "ranger": return CharacterClass.Ranger;
                case "mage": return CharacterClass.Mage;
                case "wizard": return CharacterClass.Sorcerer;
                case "elementalist": return CharacterClass.Spiritmaster;
                case "priest": return CharacterClass.Cleric;
                case "cleric": return CharacterClass.Priest;
                case "chanter": return CharacterClass.Chanter;
                default: return CharacterClass.None;
            }
        }
    }

    public enum Race
    {
	    Elyos = 0,
	    Asmodians,
        Lycan,
        Construct,
        Carrier,
        Drakan,
        Lizardman,
        Teleporter,
        Naga,
        Brownie,
        Krall,
        Shulack,
        Barrier,
        CastleDoorElyos,
        CastleDoorAsmodians,
        CastleBoorDrakan,
        DeityElyos,
        DeityAsmodians,
        DeityDrakan,
        Outsider,
        Ratman,
        SemiHumanoid,
        Undead,
        Beast,
        MagicalMonster,
        Elemental,
        None,
        PlayableAny
    }

    [XmlType(AnonymousType=true)]
    public enum WeaponType
    {
        [XmlEnum("1h_dagger")]
        Dagger,
        [XmlEnum("1h_mace")]
        Mace,
        [XmlEnum("1h_sword")]
        Sword,
        [XmlEnum("1h_toolhoe")]
        Toolhoe,
        [XmlEnum("2h_book")]
        Book,
        [XmlEnum("2h_orb")]
        Orb,
        [XmlEnum("2h_polearm")]
        Polearm,
        [XmlEnum("2h_staff")]
        Staff,
        [XmlEnum("2h_sword")]
        Sword2H,
        [XmlEnum("2h_toolpick")]
        Toolpick,
        [XmlEnum("2h_toolrod")]
        Toolrod,
        [XmlEnum("bow")]
        Bow,
        [XmlEnum("noweapon")]
        NoWeapon
    }

    [XmlType(AnonymousType = true)]
    public enum ArmorType
    {
        None = 0,
        [XmlEnum("robe")]
        Robe,
        [XmlEnum("clothes")]
        Cloth,
        [XmlEnum("leather")]
        Leather,
        [XmlEnum("plate")]
        Plate,
        [XmlEnum("no_armor")]
        NoArmor,
        [XmlEnum("chain")]
        Chain
    }

    [XmlType(AnonymousType = true)]
    public enum ItemQuality
    {
        [XmlEnum("Common")]
        Common = 0,
        [XmlEnum("Rare")]
        Rare,
        [XmlEnum("Unique")]
        Unique,
        [XmlEnum("Legend")]
        Legendary,
        [XmlEnum("Mythic")]
        Mythic,
        [XmlEnum("Epic")]
        Epic,
        [XmlEnum("Junk")]
        Junk,
        common = Common,
        rare = Rare,
        unique = Unique,
        legend = Legendary,
        mythic = Mythic,
        epic = Epic,
        junk = Junk
    }

    [XmlType(AnonymousType = true)]
    public enum BonusApplyCondition
    {
        [XmlEnum("none")]
        None = 0,
        [XmlEnum("equip")]
        OnEquip,
        [XmlEnum("inventory")]
        OnCarry
    }

    [XmlType(AnonymousType = true)]
    public enum ItemType
    {
        [XmlEnum("normal")]
        Normal = 0,
        [XmlEnum("abyss")]
        Abyssal,
        [XmlEnum("draconic")]
        Draconic,
        [XmlEnum("devanion")]
        Daevanion
    }

    [Flags]
    [XmlType(AnonymousType = true)]
    public enum EquipmentSlot
    {
        [XmlEnum("none")]
        None = 0,
        [XmlEnum("main")]
        MainHand = 1,
        [XmlEnum("sub")]
        SubHand = 2,
        [XmlEnum("main_or_sub")]
        EitherHand = 3,
        [XmlEnum("head")]
        Head = 4,
        [XmlEnum("torso")]
        Torso = 8,
        [XmlEnum("glove")]
        Hands = 16,
        [XmlEnum("foot")]
        Feet = 32,
        EarLeft = (1 << 6),
        EarRight = (1 << 7),
        FingerLeft = (1 << 8),
        FingerRight = (1 << 9),
        [XmlEnum("right_or_left_ear")]
        EitherEar = EarLeft | EarRight,
        [XmlEnum("right_or_left_finger")]
        EitherFinger = FingerLeft | FingerRight,
        [XmlEnum("neck")]
        Neck = 1024,
        [XmlEnum("shoulder")]
        Shoulders = 2048,
        [XmlEnum("leg")]
        Legs = 4096,
        BatteryLeft = (1 << 13),
        BatteryRight = (1 << 14),
        [XmlEnum("right_or_left_battery")]
        EitherBattery = BatteryLeft | BatteryRight,
        [XmlEnum("wing")]
        Wings = 32768,
        [XmlEnum("waist")]
        Waist = 65536,
        OffMain = (1 << 17),
        OffSub = (1 << 18),
    }

    public class Stats
    {
        public static Stat ForAlias(string alias)
        {
            switch (alias.ToLower().Trim())
            {
                case "phyattack": return Stat.PhysicalAttack;
                case "maxhp": return Stat.MaxHp;
                case "hitaccuracy": return Stat.PhysicalAccuracy;
                case "critical": return Stat.PhysicalCritical;
                case "parry": return Stat.Parry;
                case "attackdelay": return Stat.AttackDelay;
                case "maxmp": return Stat.MaxMp;
                case "pvpattackratio": return Stat.PvPAttackRatio;
                case "magicalskillboost": return Stat.BoostMagicalSkill;
                case "elementaldefendfire": return Stat.ElementalResistanceFire;
                case "elementaldefendearth": return Stat.ElementalResistanceEarth;
                case "block": return Stat.Block;
                case "elementaldefendwater": return Stat.ElementalResistanceWater;
                case "elementaldefendair": return Stat.ElementalResistanceWind;
                case "magicalhitaccuracy": return Stat.MagicalAccuracy;
                case "ar_paralyze":
                case "arparalyze":
                case "paralyze_arp": return Stat.ResistanceParalyze;
                case "magicalcritical": return Stat.MagicalCritical;
                case "arsilence":
                case "silence_arp": return Stat.ResistanceSilence;
                case "boostcastingtime": return Stat.BoostCastingTime;
                case "magicalattack": return Stat.MagicalAttack;
                case "dodge": return Stat.Dodge;
                case "concentration": return Stat.Concentration;
                case "boosthate": return Stat.BoostHate;
                case "maxfp": return Stat.MaxFp;
                case "flyspeed": return Stat.FlySpeed;
                case "physicaldefend": return Stat.PhysicalDefend;
                case "magical_resist":
                case "magicalresist": return Stat.MagicalResist;
                case "pvpdefendratio": return Stat.PvPDefendRatio;
                case "speed": return Stat.Speed;
                case "physicalcriticalreducerate": return Stat.PhysicalCriticalRateReduce;
                case "magicalcriticalreducerate": return Stat.MagicalCriticalRateReduce;
                case "healskillboost": return Stat.BoostHeal;
                case "arpoison": return Stat.ResistancePoison;
                case "arbleed": return Stat.ResistanceBleed;
                case "arsleep": return Stat.ResistanceSleep;
                case "arroot": return Stat.ResistanceRoot;
                case "arblind": return Stat.ResistanceBlind;
                case "archarm": return Stat.ResistanceCharm;
                case "ardisease": return Stat.ResistanceDisease;
                case "arfear": return Stat.ResistanceFear;
                case "arcurse": return Stat.ResistanceCurse;
                case "arconfuse": return Stat.ResistanceConfuse;
                case "arstun": return Stat.ResistanceStun;
                case "arperification": return Stat.ResistancePetrification;
                case "arstumble": return Stat.ResistanceStumble;
                case "arstagger": return Stat.ResistanceStagger;
                case "aropenareial": return Stat.ResistanceOpenAerial;
                case "arsnare": return Stat.ResistanceSnare;
                case "arslow": return Stat.ResistanceSlow;
                case "arspin": return Stat.ResistanceSpin;
                case "magicalcriticaldamagereduce": return Stat.MagicalCriticalDamageReduce;
                case "physicalcriticaldamagereduce": return Stat.PhysicalCriticalDamageReduce;
                default: Log.Warn("Unknown stat alias '" + alias.ToLower().Trim() + "'"); Console.ReadLine(); return Stat.None;
            }
        }
    }

    public enum Stat
    {
        None = 0,
        Accuracy,
        Agility,
        AllAbnormals,
        Attack,
        AttackRange,
        AttackDelay,
        Block,
        BoostCastingTime,
        BoostHate,
        BoostHeal,
        BoostMagicalSkill,
        BoostStagger,
        BoostStumble,
        BoostStun,
        Concentration,
        DamageReduce,
        ElementalResistanceDark,
        ElementalResistanceEarth,
        ElementalResistanceFire,
        ElementalResistanceLight,
        ElementalResistanceWater,
        ElementalResistanceWind,
        Dodge,
        FlySpeed,
        MaxFp,
        Health,
        HitCount,
        Knowledge,
        MagicalAccuracy,
        MagicalAttack,
        MagicalCritical,
        MagicalCriticalDamageReduce,
        MagicalCriticalRateReduce,
        MagicalCriticalResist,
        MagicalDefend,
        MagicalResist,
        MainHandAccuracy,
        MainHandCritical,
        MainHandAttack,
        MaxDamages,
        MaxDp,
        MaxHp,
        MaxMp,
        MinDamages,
        NaturalHealingFp,
        NaturalHealingHp,
        NaturalHealingMp,
        OffHandAccuracy,
        OffHandCritical,
        OffHandAttack,
        Parry,
        PhysicalAccuracy,
        PhysicalAttack,
        PhysicalCritical,
        PhysicalCriticalRateReduce,
        PhysicalCriticalDamageReduce,
        PhysicalCriticalResist,
        PhysicalDefend,
        PvPAttackRatio,
        PvPDefendRatio,
        ResistanceAll,
        ResistanceBleed,
        ResistanceBlind,
        ResistanceCharm,
        ResistanceConfuse,
        ResistanceCurse,
        ResistanceDisease,
        ResistanceEarth,
        ResistanceFear,
        ResistanceFire,
        ResistanceWater,
        ResistanceWind,
        ResistanceOpenAerial,
        ResistanceParalyze,
        ResistancePetrification,
        ResistancePoison,
        ResistanceRoot,
        ResistanceSilence,
        ResistanceSleep,
        ResistanceSlow,
        ResistanceSnare,
        ResistanceSpin,
        ResistanceStagger,
        ResistanceStumble,
        ResistanceStun,
        ResistanceTransform,
        Speed,
        Will
    }

    public enum CharacterCreateResponse
    {
        Success = 0,
        Failed = 1,
        DatabaseError = 2,
        LimitExceeded = 4,
        InvalidName = 5,
        ForbiddenWords = 9,
        NameAlreadyUsed = 10,
        NameReserved = 11,
        WrongRace = 12
    }

    public enum StorageType
    {
        Cube = 0,
        Warehouse,
        AccountWarehouse,
        LegionWarehouse,
        PetBag6Slot,
        PetBag12Slot,
        PetBag18Slot,
        PetBag24Slot,
        Broker,
        Mailbox,
    }

    [XmlType(AnonymousType = true)]
    public enum AttackType
    {
        [XmlEnum("physical")]
        Physical,
        [XmlEnum("magical")]
        Magical,
        [XmlEnum("magical_fire")]
        MagicalFire,
        [XmlEnum("magical_water")]
        MagicalWater,
    }

    [XmlType(AnonymousType=true)]
    public enum GenderPermitted
    {
        [XmlEnum("male")]
        Male = 0,
        [XmlEnum("female")]
        Female,
        [XmlEnum("all")]
        All
    }

    [XmlType(AnonymousType = true)]
    public enum StigmaType
    {
        [XmlEnum("Enhanced1")]
        Enhanced
    }

    [XmlType(AnonymousType = true)]
    public enum SubEnchantMaterialEffectType
    {
        Probability = 0,
        [XmlEnum("cash_option_prob")]
        CashOptionProbability
    }

    [XmlType(AnonymousType = true)]
    public enum EffectType
    {
        [XmlEnum("HolyOffenseProvokerEffect")]
        HolyOffenseProvoker,
    }

    [Flags]
    public enum ItemPermission
    {
        Lore = 1,
        CanExchange = (1 << 1),
        CanSellToNpc = (1 << 2),
        CanStore_CharWH = (1 << 3),
        CanStore_AccountWH = (1 << 4),
        CanStore_LegionWH = (1 << 5),
        Breakable = (1 << 6),
        SoulBindRequired = (1 << 7),
        RemoveWhenLogout = (1 << 8),
        CanEnchant = (1 << 9),
        CanProcEnchant = (1 << 10),
        CanCompositeWeapon = (1 << 11),
        CanChangeSkin = (1 << 12),
        CanSplitWeapon = (1 << 13),
        Deletable = (1 << 14),
        CanDye = (1 << 15)
    }

    public enum StorageDisplayType
    {
        None = 1,
        Drop = 17,
        NewItem = 25
    }
    
    public enum MovementType : byte
    {
        ElevatorJump = 208,
        ElevatorMove = 240,
        ElevatorStand = 24,
        ElevatorOn = 16,
        GlideDown = 196,
        GlideMoveStart = 228,
        GlideMove = 164,
        GlideUp = 132,
        StartKeyboard = 192,
        StartMouse = 224,
        Stop = 0,
        ValidationElevator = 144,
        ValidationJump = 8,
        ValidationJumpInMove = 72,
        ValidationKeyboard = 128,
        ValidationMouse = 160,
    }

    public enum Levels
    {
        // Base Atreia.
        Altgard = 220030000,
        Beluslan = 220040000,
        Brusthonin = 220050000,
        Eltnen = 210020000,
        Gelkmaros = 220070000,
        Heiron = 210020000,
        Inggison = 210050000,
        Ishalgen = 220010000,
        Morheim = 220020000,
        Pandaemonium = 120010000,
        Poeta = 210010000,
        PrisonLight = 510010000,
        PrisonDark = 520010000,
        Reshanta = 400010000,
        Sanctum = 110010000,
        Silentera = 600010000,
        Theobomos = 210060000,
        Verteron = 210030000,
    }

    public enum PublicMessageType
    {
        Regular = 0,
        Shout = 3,
        Whisper = 4,
        Group = 5,
        Alliance = 6,
        GroupLeader = 7,
        Legion = 10,
        SystemYellow = 0x19,
        SystemWhite = 0x20,
        SystemAlert = 0x22
    }

    public enum ItemError : byte
    {
        None = 0,
        NoSpaceLeft,
    }

    public enum SkillType : byte
    {
        Physical = 0,
        Magical,
    }

    public enum SkillSubType : byte
    {
        None = 0,
        none = None,
        Attack,
        attack = Attack,
        Buff,
        buff = Buff,
        Chant,
        Debuff,
        debuff = Debuff,
        Heal,
        heal = Heal,
        Summon,
        summon = Summon,
        SummonTrap,
        SummonHoming,
    }

    public enum ActivationAttribute : byte
    {
        Passive = 0,
        Active,
        Toggle,
        Maintain,
        [XmlEnum("maintain")]
        __maintain = Maintain,
        Provoked,
    }

    public enum SkillTargetSlot : byte
    {
        NoShow = 0,
        [XmlEnum("Noshow")]
        __Noshow = NoShow,
        Boost,
        Buff,
        [XmlEnum("buff")]
        __buff = Buff,
        Chant,
        Debuff,
        [XmlEnum("debuff")]
        __debuff = Debuff,
        [XmlEnum("DeBuff")]
        __DeBuff = Debuff,
        None,
        [XmlEnum("none")]
        __none = None,
        Special,
        [XmlEnum("special2")]
        Special2,
    }

    public enum FirstTarget : byte
    {
        Me = 0,
        [XmlEnum("ME")]
        __ME = Me,
        [XmlEnum("me")]
        __me = Me,
        Target,
        [XmlEnum("target")]
        __target = Target,
        TargetOrMe,
        [XmlEnum("TargetorMe")]
        __TargetorMe = TargetOrMe,
        Point,
        MyPet,
        Target_MyParty_NonVisible,
    }

    public enum SkillEffectTargetType : byte
    {
        Target_Only,
    }

    public enum SkillEffectType : byte
    {
        AbsoluteSnare,
        AbsoluteSlow,
        Activate_Enslave,
        AlwaysBlock,
        AlwaysDodge,
        AlwaysParry,
        AlwaysResist,
        Amr_Mastery,
        Aura,
        BackDashATK,
        Bind,
        Bleed,
        Blind,
        BoostDropRate,
        BoostHate,
        BoostHealEffect,
        BoostSkillCastingTime,
        BoostSkillCost,
        BoostSpellAttackEffect,
        BuffBind,
        BuffSilence,
        BuffSleep,
        BuffStun,
        Cannon,
        CarveSignet,
        ChangeHateOnAttacked,
        ConvertHeal,
        CloseAerial,
        CondSkillLauncher,
        Confuse,
        Curse,
        DashATK,
        DeathBlow,
        DeBoostHealAmount,
        Deform,
        DelayedFPATK_Instant,
        DelayedSkill,
        DelayedSpellATK_Instant,
        Disease,
        Dispel,
        DispelBuff,
        DispelBuffCounterATK,
        DispelDeBuff,
        DispelDebuffPhysical,
        DispelDebuffMental,
        DispelNpcBuff,
        DispelNPCDebuff,
        DPHeal_Instant,
        DPHeal,
        DPTransfer,
        DRBoost,
        Dummy,
        Evade,
        ExtendAuraRange,
        Fall,
        Fear,
        Flyoff,
        FPATK,
        FPATK_Instant,
        FPHeal,
        FPHeal_Instant,
        Heal,
        HealCastorOnAttacked,
        HealCastorOnTargetDead,
        Heal_Instant,
        Hide,
        HiPass,
        HostileUp,
        IntervalSkill,
        InvulnerableWing,
        MagicCounterATK,
        MoveBehindATK,
        MPAttack,
        MPAttack_Instant,
        MPHeal,
        MPHeal_Instant,
        NoFly,
        NoReduceSpellATK_Instant,
        NoDeathPenalty,
        NoResurrectPenalty,
        OneTimeBoostHealEffect,
        OneTimeBoostSkillAttack,
        OneTimeBoostSkillCritical,
        OpenAerial,
        Paralyze,
        PetOrderUseUltraSkill,
        Petrification,
        Poison,
        Polymorph,
        Protect,
        ProcATK_Instant,
        ProcDPHeal_Instant,
        ProcHeal_Instant,
        ProcFpHeal_Instant,
        ProcMPHeal_Instant,
        Provoker,
        Pulled,
        RandomMoveLoc,
        Rebirth,
        Recall_Instant,
        Reflector,
        Resurrect,
        ResurrectBase,
        ResurrectPositional,
        ReturnHome,
        ReturnPoint,
        Root,
        Sanctuary,
        Search,
        Signet,
        Sleep,
        ShieldMastery,
        SkillATKDrain_Instant,
        SkillATK_Instant,
        SkillLauncher,
        SpellATK,
        SpellATKDrain,
        SpellATKDrain_Instant,
        SpellATK_Instant,
        ShapeChange,
        Shield,
        SignetBurst,
        Silence,
        Simple_Root,
        Slow,
        Snare,
        Spin,
        Stagger,
        StatBoost,
        StatDown,
        StatUp,
        Stumble,
        Stun,
        SubTypeBoostResist,
        SubTypeExtendDuration,
        Summon,
        SummonBindingGroupGate,
        SummonGroupGate,
        SummonHoming,
        SummonServant,
        SummonSkillArea,
        SummonTotem,
        SummonTrap,
        SwitchHostile,
        SwitchHPMP_Instant,
        TargetTeleport,
        WeaponStatBoost,
        WeaponStatUp,
        Wpn_Dual,
        Wpn_Mastery,
        XPBoost,
    }

    [Flags]
    public enum AlteredState
    {
        Buff = 0,
        Poison = 1,
        Bleed = 2,
        Paralyze = 4,
        Sleep = 8,
        Root = 16,
        Blind = 32,
        Unk = 64,
        Disease = 128,
        Silence = 256,
        Fear = 512,
        Curse = 1024,
        Chaos = 2048,
        Stun = 4096,
        Petrification = 8192,
        Stumble = 16384,
        Stagger = 32768,
        OpenAerial = 65536,
        Snare = 131072,
        Slow = 262144,
        Spin = 524288,
        Bind = 1048576,
        Deform = 2097152,
        CannotMove = 4194304,
        CannotFly = 8388608,
        Knockback = 16777216,
        Invisible = 536870912,
        _flying,
        _Nflying,
    }

    public enum SkillEffectHopType : byte
    {
        Damage,
        SkillLv,
    }

    public enum EmotionType : byte
    {
        SelectTarget = 0,
        Jump = 1,
        Sit = 2,
        Stand = 3,
        Chair_Sit = 4,
        Chair_Up = 5,
        Airline_Start = 6,
        Airline_Stop = 7,
        Windstream_Start = 8,
        Windstream_Stop = 9,
        Windstream_Boost = 11,
        Fly = 13,
        Land = 14,
        Death = 18,
        Resurrect = 19,
        SocialEmote = 21,
        Duel_End = 22,
        AttackMode = 24,
        AttackMode2 = 38,
        RegularMode = 25,
        Walk = 26,
        Run = 27,
        SwitchDoor = 31,
        Emote_Start = 32,
        Emote_Start2 = 35,
        PrivateShop_Start = 33,
        PrivateShop_Stop = 34,
        Powershard_Enable = 36,
        Powershard_Disable = 37,
        Loot_Start = 40,
        Loot_End = 41,
        QuestLoot_Start = 42,
        QuestLoot_End = 43,
        ToyPetFeed = 50,
        ToyPetFeed2 = 51,
    }

}
