﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace AionCxxLibrary.Packets
{
    public abstract class AionClientPacket : AionPacket
    {
        public AionClientPacket() : base(PacketType.CLIENT) { }

        public abstract void ProcessPacket();

        protected byte[] ReadBytes(int length)
        {
            byte[] result = new byte[length];
            Read(result, 0, length);
            return result;
        }

        protected new byte ReadByte()
        {
            return ReadBytes(1)[0];
        }

        protected int ReadInt()
        {
            return BitConverter.ToInt32(ReadBytes(4), 0);
        }

        protected float ReadFloat()
        {
            return BitConverter.ToSingle(ReadBytes(4), 0);
        }

        protected short ReadShort()
        {
            return BitConverter.ToInt16(ReadBytes(2), 0);
        }

        protected ushort ReadUnsignedShort()
        {
            return BitConverter.ToUInt16(ReadBytes(2), 0);
        }

        protected string ReadString()
        {
            string result = "";
            // loop the stream until end, will be broken in middle if string end found
            while ((Length - Position) > 2)
            {
                char c = BitConverter.ToChar(ReadBytes(2), 0);
                if (c == 0)
                    break;
                else
                    result += c;
            }
            return result;
        }

    }
}
