﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using System.Configuration;
using AionCxxLibrary.Utilities;

namespace WorldServer.DAO
{
    public class RecipeDAO
    {
        public static List<int> SelectRecipesByCharacterID(int characterId)
        {
            SqlConnection con = new SqlConnection(ConfigurationManager.ConnectionStrings["WorldCatalog"].ToString());
            SqlCommand cmd = new SqlCommand("SELECT RecipeID FROM dbo.Recipes WHERE CharacterID = @CharID", con);
            cmd.Parameters.Add("@CharID", System.Data.SqlDbType.Int).Value = characterId;
            // add command params here
            List<int> result = new List<int>();
            Log.Query(cmd);
            try
            {
                con.Open();
                SqlDataReader rdr = cmd.ExecuteReader();
                while (rdr.Read())
                {
                    int recipeid = rdr.GetInt32(0);
                    result.Add(recipeid);
                }
            }
            catch (Exception e)
            {
                Log.Error("Cannot load recipes for " + characterId, e);
                result.Clear();
            }
            finally
            {
                cmd.Dispose();
                con.Close();
            }
            return result;
        }

        public static void InsertRecipe(int charId, int recipeId)
        {
            SqlConnection con = new SqlConnection(ConfigurationManager.ConnectionStrings["WorldCatalog"].ToString());
            SqlCommand cmd = new SqlCommand("INSERT INTO dbo.Recipes(CharacterID, RecipeID) VALUES(@CharID, @RecipeID)", con);
            cmd.Parameters.Add("@CharID", System.Data.SqlDbType.Int).Value = charId;
            cmd.Parameters.Add("@RecipeID", System.Data.SqlDbType.Int).Value = recipeId;
            // add command params here*
            Log.Query(cmd);
            try
            {
                con.Open();
                cmd.ExecuteNonQuery();
            }
            catch (Exception e)
            {
                Log.Error("Cannot insert recipe " + recipeId + " for character " + charId, e);
            }
            finally
            {
                cmd.Dispose();
                con.Close();
            }
        }

    }
}
