﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using System.Configuration;
using System.Data;
using AionCxxLibrary.Utilities;
using AionCxxLibrary.Models;
using AionCxxLibrary.Enums;
using WorldServer.Models;

namespace WorldServer.DAO
{
    public class SkillsDAO
    {
        public static List<SkillEntry> SelectSkillsByCharacterID(int characterId)
        {
            SqlConnection con = new SqlConnection(ConfigurationManager.ConnectionStrings["WorldCatalog"].ToString());
            SqlCommand cmd = new SqlCommand("SELECT SkillID, SkillLevel FROM dbo.Skills WHERE CharacterID = @CharID", con);
            cmd.Parameters.Add("@CharID", SqlDbType.Int).Value = characterId;
            List<SkillEntry> result = new List<SkillEntry>();
            Log.Query(cmd);
            try
            {
                con.Open();
                SqlDataReader rdr = cmd.ExecuteReader();
                while (rdr.Read())
                {
                    result.Add(new SkillEntry() { SkillID = rdr.GetInt32(0), Level = rdr.GetByte(1) });
                }
            }
            catch (Exception e)
            {
                Log.Error("Cannot load skills for " + characterId, e);
                result.Clear();
            }
            finally
            {
                cmd.Dispose();
                con.Close();
            }
            return result;
        }

        public static void DeleteSkill(int characterId, int skillId)
        {
            SqlConnection con = new SqlConnection(ConfigurationManager.ConnectionStrings["WorldCatalog"].ToString());
            SqlCommand cmd = new SqlCommand("DELETE FROM dbo.Skills WHERE CharacterID = @CharID AND SkillID = @SkillID", con);
            cmd.Parameters.Add("@CharID", SqlDbType.Int).Value = characterId;
            cmd.Parameters.Add("@SkillID", SqlDbType.Int).Value = skillId;
            Log.Query(cmd);
            try
            {
                con.Open();
                cmd.ExecuteNonQuery();
            }
            catch (Exception e)
            {
                Log.Error("", e);
            }
            finally
            {
                cmd.Dispose();
                con.Close();
            }
        }

        public static void InsertSkill(int characterId, int skillId, int level)
        {
            SqlConnection con = new SqlConnection(ConfigurationManager.ConnectionStrings["WorldCatalog"].ToString());
            SqlCommand cmd = new SqlCommand("INSERT INTO dbo.Skills(CharacterID, SkillID, SkillLevel) VALUES(@CharID, @SkillID, @SkillLevel)", con);
            cmd.Parameters.Add("@CharID", SqlDbType.Int).Value = characterId;
            cmd.Parameters.Add("@SkillID", SqlDbType.Int).Value = skillId;
            cmd.Parameters.Add("@SkillLevel", SqlDbType.TinyInt).Value = level;
            SqlTransaction transac = null;
            Log.Query(cmd);
            try
            {
                con.Open();
                transac = con.BeginTransaction();
                cmd.Transaction = transac;
                cmd.ExecuteNonQuery();
                transac.Commit();
            }
            catch (Exception e)
            {
                Log.Error("Cannot insert skill " + skillId + "/" + level + " for character " + characterId, e);
                transac.Rollback();
            }
            finally
            {
                cmd.Dispose();
                con.Close();
            }
        }

        public static void UpdateSkillLevel(int characterId, int skillId, int level)
        {
            SqlConnection con = new SqlConnection(ConfigurationManager.ConnectionStrings["WorldCatalog"].ToString());
            SqlCommand cmd = new SqlCommand("UPDATE dbo.Skills SET SkillLevel = @SkillLevel WHERE CharacterID = @CharID and SkillID = @SkillID", con);
            // add command params here
            cmd.Parameters.Add("@CharID", SqlDbType.Int).Value = characterId;
            cmd.Parameters.Add("@SkillID", SqlDbType.Int).Value = skillId;
            cmd.Parameters.Add("@SkillLevel", SqlDbType.TinyInt).Value = level;
            Log.Query(cmd);
            try
            {
                con.Open();
                cmd.ExecuteNonQuery();
            }
            catch (Exception e)
            {
                Log.Error("Cannot update skill " + skillId + " level to " + level + " for character " + characterId, e);
            }
            finally
            {
                cmd.Dispose();
                con.Close();
            }
        }

    }
}
