﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WorldServer.DAO;

namespace WorldServer.Models
{
    /// <summary>
    /// This class represents the list of character IDs in the friend list of a given character.
    /// </summary>
    public class FriendsList : EventAwareList<int>
    {
        Character c;
        public FriendsList(Character c)
        {
            this.c = c;
            AddRange(FriendsDAO.SelectFriends(c.GetGUID()));
            OnAdd += new EventAwareListAddEvent<int>(FriendsList_OnAdd);
            OnRemove += new EventAwareListRemoveEvent<int>(FriendsList_OnRemove);
        }

        void FriendsList_OnRemove(object sender, EventArgs<int> e)
        {
            FriendsDAO.InsertFriend(c.GetGUID(), e.Value);
        }

        void FriendsList_OnAdd(object sender, EventArgs<int> e)
        {
            FriendsDAO.RemoveFriend(c.GetGUID(), e.Value);
        }
    }

    public class BlockList
    {
        Character c;
        public Dictionary<string, string> List = new Dictionary<string, string>();

        public BlockList(Character c)
        {
            this.c = c;
            List = BlocksDAO.SelectBlocks(c.GetGUID());
        }

        void RemoveBlock(string name)
        {
            List.Remove(name);
            BlocksDAO.RemoveBlock(c.GetGUID(), name);
        }

        void InsertBlock(string name, string reason)
        {
            List.Add(name, reason);
            BlocksDAO.InsertBlock(c.GetGUID(), name, reason);
        }
    }

}
