﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using AionCxxLibrary.Packets;
using WorldServer.Services;
using WorldServer.Models;
using AionCxxLibrary.Utilities;
using AionCxxLibrary.Enums;
using WorldServer.Packets.Server;

namespace WorldServer.Packets.Client
{
    public class CPkt_LevelLoaded : AionClientPacket
    {
        public override void ProcessPacket()
        {
            CharacterService.OnEnterLevel(((WorldConnection)Connection).Character);
        }
    }

    public class CPkt_CharacterMove : AionClientPacket
    {
        public override void ProcessPacket()
        {
            byte[] data = ReadBytes((int)(Length - Position));

            float x = BitConverter.ToSingle(data, 0);
            float y = BitConverter.ToSingle(data, 4);
            float z = BitConverter.ToSingle(data, 8);

            float x2 = 0f;
            float y2 = 0f;
            float z2 = 0f;

            byte glideFlag = 0;

            byte heading = data[12];
            MovementType moveType;
            byte moveId = data[13];

            if (Enum.IsDefined(typeof(MovementType), moveId))
            {
                moveType = (MovementType)moveId;
            }
            else
            {
                Log.Debug("Unknown move type " + moveId);
                return;
            }

            if (data.Length > 22)
            {
                x2 = BitConverter.ToSingle(data, 14);
                y2 = BitConverter.ToSingle(data, 18);
                z2 = BitConverter.ToSingle(data, 22);
            }

            if (data.Length == 15)
            {
                glideFlag = data[14];
            }
            else if (data.Length > 26)
            {
                glideFlag = data[26];
            }

            Character c = ((WorldConnection)Connection).Character;

            LevelService.Move(c, moveType, x, y, z, x2, y2, z2, heading);
         
            c.NearbyObjects.BroadcastPacket(new SPkt_ObjectMove(c.GetGUID(), data), false);

        }
    }

}
