﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using AionCxxLibrary.Packets;
using AionCxxLibrary.Utilities;
using WorldServer.Services;
using WorldServer.Models;
using WorldServer.Packets.Server;
using AionCxxLibrary.Enums;
using WorldServer.Helpers;

namespace WorldServer.Packets.Client
{
    public class CPkt_SocialStatusUpdate : AionClientPacket
    {
        public override void ProcessPacket()
        {
            ((WorldConnection)Connection).Character.SocialState = ReadByte();
        }
    }

    public class CPkt_SocialFriendsRequest : AionClientPacket
    {
        public override void ProcessPacket()
        {
            Connection.SendPacket(new SPkt_SocialFriends());
        }
    }

    public class CPkt_SocialBlocksRequest : AionClientPacket
    {
        public override void ProcessPacket()
        {
            Connection.SendPacket(new SPkt_SocialBlocks());
        }
    }

    public class CPkt_SocialPublicMessage : AionClientPacket
    {
        public override void ProcessPacket()
        {
            PublicMessageType type = (PublicMessageType)(int)ReadByte();
            string message = ReadString();
            SocialService.BroadcastPublicMessage((Connection as WorldConnection).Character, type, message);
        }
    }

    public class CPkt_SocialPrivateMessage : AionClientPacket
    {
        public override void ProcessPacket()
        {
            
        }
    }

    public class CPkt_Emotion : AionClientPacket
    {
        public override void ProcessPacket()
        {
            EmotionType etype = (EmotionType)ReadByte();
            ushort emoteId = 0;
            float x = 0, y = 0, z = 0;
            byte heading = 0;
            switch (etype)
            {
                case EmotionType.SocialEmote:
                    emoteId = ReadUnsignedShort();
                    break;
                case EmotionType.Chair_Sit:
                case EmotionType.Chair_Up:
                    x = ReadFloat();
                    y = ReadFloat();
                    z = ReadFloat();
                    heading = ReadByte();
                    break;
            }
            Character ch = (Connection as WorldConnection).Character;
            ch.NearbyObjects.BroadcastPacket(new SPkt_Emotion(ch.GetGUID(), etype, x, y, z, heading, ch.Target != null ? ch.Target.GetGUID() : 0, emoteId), true);
        }
    }

}
