﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Data.SqlClient;

namespace AionCxxLibrary.Utilities
{
    public class Log
    {
        static StreamWriter writer;

        public static void Initialize(string filename)
        {
            try
            {
                writer = new StreamWriter(filename, true);
            }
            catch (Exception e)
            {
                Console.WriteLine("Cannot initialize the Log!");
                Console.WriteLine(e.Message);
                Console.WriteLine(e.StackTrace);
            }
        }

        static void AppendLog(string message)
        {
            if (writer == null) return;
            string result = DateTime.Now.ToString() + " " + message;
            lock (writer)
            {
                writer.WriteLine(message);
                writer.Flush();
            }
        }

        static object console_mutex = new object();

        static void OutputColor(string message, ConsoleColor clr)
        {
            lock (console_mutex)
            {
                ConsoleColor old = Console.ForegroundColor;
                Console.ForegroundColor = clr;
                Console.WriteLine(DateTime.Now.ToString() + " " + message);
                Console.ForegroundColor = old;
            }
        }

        public static void Info(string message)
        {
            string result = "[INFO] " + message;
            AppendLog(result);
            OutputColor(result, ConsoleColor.Green);
        }

        public static void Hack(string message)
        {
            string result = "[HACK] " + message;
            AppendLog(result);
            OutputColor(result, ConsoleColor.Blue);
        }

        public static void Error(string message)
        {
            Error(message, null);
        }

        public static void Error(string message, Exception e)
        {
            string result = "[ERROR] " + message;
            if(e != null)
                result += "\n" + e.Message + "\n" + e.StackTrace;
            AppendLog(result);
            OutputColor(result, ConsoleColor.Red);
        }

        public static void Warn(string message)
        {
            Warn(message, null);
        }

        public static void Warn(string message, Exception e)
        {
            string result = "[WARN] " + message;
            if (e != null)
                result += "\n" + e.Message + "\n" + e.StackTrace;
            AppendLog(result);
            OutputColor(result, ConsoleColor.Yellow);
        }

        public static void Debug(string message)
        {
            string result = "[DEBUG] " + message;
            AppendLog(result);
            OutputColor(result, ConsoleColor.White);
        }

        public static void Query(SqlCommand query)
        {
            string q = query.CommandText;
            foreach(SqlParameter param in query.Parameters)
            {
                bool isstr = param.SqlDbType == System.Data.SqlDbType.VarChar;
                q = q.Replace(param.ParameterName, (isstr ? "'" : "") + param.Value.ToString() + (isstr ? "'" : ""));
            }
            string result = "[QUERY] " + q;
            AppendLog(result);
            OutputColor(result, ConsoleColor.DarkMagenta);
        }

        public static void Chat(string message)
        {
            string result = "[CHAT] " + message;
            AppendLog(result);
            OutputColor(result, ConsoleColor.Cyan);
        }

    }
}
