﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using AionCxxLibrary.Packets;
using AionCxxLibrary.Utilities;
using AionCxxLibrary.Enums;
using AuthenticationServer.Services;
using AionCxxLibrary.DAO;

namespace AuthenticationServer.Packets
{
    public class CPkt_GameGuard : AionClientPacket
    {
        public override void ProcessPacket()
        {
            int sessionID = ReadInt();
            if (sessionID != Connection.GetHashCode())
            {
                Log.Hack("GameGuard SessionID != Connection.HashCode");
                Connection.SendPacket(new SPkt_LoginFailed(AuthenticationResponse.NoInfoAvailable));
                return;
            }
            Connection.SendPacket(new SPkt_GameGuard());
        }
    }

    public class CPkt_Login : AionClientPacket
    {
        public override void ProcessPacket()
        {
            ReadInt();
            byte[] data = new byte[128];
            Read(data, 0, 128);

            byte[] decrypted = ((AuthenticationCrypter)Connection.Crypter).Rsa.PrivateDecrypt(data, OpenSSL.Crypto.RSA.Padding.None);

            string AccountName = Encoding.UTF8.GetString(decrypted, 64, 32).Trim().Replace("\0", "").ToUpper();
            string AccountPassword = Encoding.UTF8.GetString(decrypted, 96, 32).Trim().Replace("\0", "");
            
            // if login ok
            int AccountID = AccountDAO.Authenticate(AccountName, AccountPassword);
            if(AccountID == 0)
            {
                Log.Warn("Account '" + AccountName + "': Invalid username or password.");
                Connection.SendPacket(new SPkt_LoginFailed(AuthenticationResponse.InvalidPassword));
                return;
            }
            
            // Generate a triplet of integral keys, one for login and 2 for world.
            Random rnd = new Random();
            AuthenticationConnection con = (AuthenticationConnection)Connection;
            con.AccountID = AccountID;
            con.Key_Login = rnd.Next();
            con.Key_World1 = rnd.Next();
            con.Key_World2 = rnd.Next();

            AccountDAO.ClearSessionsForAccount(AccountID);
            AccountDAO.UpdateSession(AccountID, con.Key_Login, con.Key_World1, con.Key_World2, 0);

            Connection.SendPacket(new SPkt_Login());
        }
    }

    public class CPkt_RealmList : AionClientPacket
    {
        public override void ProcessPacket()
        {
            int AccountID = ReadInt();
            int KeyLogin = ReadInt();
            AuthenticationConnection con = (AuthenticationConnection)Connection;
            if(AccountID != con.AccountID || KeyLogin != con.Key_Login)
            {
                Connection.SendPacket(new SPkt_LoginFailed(AuthenticationResponse.LoadAcctInfoFailed));
                return;
            }
            Connection.SendPacket(new SPkt_RealmList());
        }
    }

    public class CPkt_JoinWorld : AionClientPacket
    {
        public override void ProcessPacket()
        {
            int AccountID = ReadInt();
            int KeyLogin = ReadInt();
            int ServerID = ReadByte();
            AuthenticationConnection con = (AuthenticationConnection)Connection;

            if (AccountID != con.AccountID || KeyLogin != con.Key_Login)
            {
                Connection.SendPacket(new SPkt_JoinWorldFailed(AuthenticationResponse.LoadAcctInfoFailed));
                return;
            }

            Connection.SendPacket(new SPkt_JoinWorld(ServerID));
            
        }
    }

    public class CPkt_Reconnect : AionClientPacket
    {
        public override void ProcessPacket()
        {
            int AccountID = ReadInt();
            int LoginOK = ReadInt();
            int ReconnectKey = ReadInt();

            if (AccountDAO.CheckSession(AccountID, LoginOK, ReconnectKey))
            {
                ((AuthenticationConnection)Connection).AccountID = AccountID;
                ((AuthenticationConnection)Connection).Key_Login = LoginOK;
                Connection.SendPacket(new SPkt_Reconnect());
            }
            else
            {
                Log.Hack(Connection.GetRemoteIP() + " has invalid reconnection tokens " + AccountID + "/" + LoginOK + "/" + ReconnectKey);
                Connection.Close();
            }

        }
    }

}
