﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net.Sockets;
using System.Net;
using System.Threading;
using AionCxxLibrary.Network;
using AionCxxLibrary.Utilities;
using AionCxxLibrary.Packets;
using AuthenticationServer.Packets;
using System.Security.Cryptography;
using Org.BouncyCastle.Crypto.Engines;
using Org.BouncyCastle.Crypto.Parameters;

namespace AuthenticationServer.Services
{
    public class AuthenticationConnection : AionConnection
    {
        public int Key_Login;
        public int Key_World1;
        public int Key_World2;
        public int AccountID;

        public AuthenticationConnection(TcpClient client) : base(client, new AuthenticationCrypter())
        {
            
        }

        public override void OnConnect()
        {
            base.OnConnect();
            // Send Init Packet.
            Log.Debug("New connection from " + GetRemoteIP());
            BeginWrite(new SPkt_Init());
        }
    }    

    public class AuthenticationListener : TcpListener
    {
        public AuthenticationListener()
            : base(IPAddress.Parse("0.0.0.0"), 2106)
        {

        }
    }

    public class NetworkService
    {
        static AuthenticationListener Listener;

        public static void Start()
        {
            AionCxxLibrary.Packets.Packets.instance = new AuthenticationPackets();
            AionCxxLibrary.Packets.Packets.instance.Initialize();

            Listener = new AuthenticationListener();
            Listener.Start();
            BeginAccept();
        }

        static AuthenticationConnection CnxPreAlloc;

        static void BeginAccept()
        {
            CnxPreAlloc = new AuthenticationConnection(null);
            Listener.BeginAcceptTcpClient(new AsyncCallback(EndAccept), Listener);
        }

        static void EndAccept(IAsyncResult ar)
        {
            TcpListener lsn = (TcpListener)ar.AsyncState;
            TcpClient client = lsn.EndAcceptTcpClient(ar);

            CnxPreAlloc.SetClient(client);
            CnxPreAlloc.OnConnect();

            BeginAccept();
        }

    }
}
