﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WorldServer.Models;

namespace WorldServer.Commands
{
    public class CxxSyntax : System.Attribute
    {
        private string syntax;
        public CxxSyntax(string syntax)
        {
            this.syntax = syntax;
        }
        public string GetSyntax()
        {
            return syntax;
        }
    }

    public class SyntaxException : System.Exception
    {
        
    }

    public abstract class CxxCommand<T> where T : CxxCommand<T>, new()
    {
        public static void ExecuteCommand(Character sender, string[] parameters)
        {
            T cmd = new T();
            try
            {
                string[] p = new string[parameters.Length - 1];
                for (int i = 1; i < parameters.Length; i++)
                    p[i - 1] = parameters[i];
                cmd.Execute(sender, p);
            }
            catch (Exception e)
            {
                if (e is SyntaxException)
                {
                    System.Attribute syntax = System.Attribute.GetCustomAttribute(cmd.GetType(), typeof(CxxSyntax));
                    if (syntax == null)
                        sender.SendMessage("Syntax Error. Sorry, not further help programmed ... :(");
                    else
                        sender.SendMessage((syntax as CxxSyntax).GetSyntax());
                }
                else
                {
                    sender.SendMessage("ERROR: " + e.Message + "\n" + e.StackTrace);
                }
            }
        }
        protected abstract void Execute(Character sender, string[] parameters);
    }

    public class CxxCommands
    {
        public static bool Execute(Character sender, string msg)
        {
            string[] splitted = msg.Split(' ');
            if (!splitted[0].StartsWith("."))
                return false;
            switch (splitted[0].Substring(1))
            {
                case "add":
                case "additem":
                    CxxCommand<Add>.ExecuteCommand(sender, splitted);
                    break;
                case "debug":
                    CxxCommand<Debug>.ExecuteCommand(sender, splitted);
                    break;
                case "gob":
                case "gameobject":
                    CxxCommand<GameObject>.ExecuteCommand(sender, splitted);
                    break;
                case "learn":
                case "skill":
                    CxxCommand<Learn>.ExecuteCommand(sender, splitted);
                    break;
                case "set":
                    CxxCommand<Set>.ExecuteCommand(sender, splitted);
                    break;
                default: 
                    sender.SendMessage("There is no such command: " + splitted[0]); 
                    break;
            }
            return true;
        }
    }

}
