﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using System.Configuration;
using AionCxxLibrary.Utilities;

namespace WorldServer.DAO
{
    public class BlocksDAO
    {
        public static Dictionary<string,string> SelectBlocks(int characterId)
        {
            SqlConnection con = new SqlConnection(ConfigurationManager.ConnectionStrings["WorldCatalog"].ToString());
            SqlCommand cmd = new SqlCommand("SELECT BlockedName, Reason FROM dbo.Blocks WHERE CharacterID = @CharID", con);
            cmd.Parameters.Add("@CharID", System.Data.SqlDbType.Int).Value = characterId;
            Dictionary<string, string> result = new Dictionary<string, string>();
            Log.Query(cmd);
            // add command params here
            try
            {
                con.Open();
                SqlDataReader rdr = cmd.ExecuteReader();
                while (rdr.Read())
                {
                    result.Add(rdr.GetString(0), rdr.GetString(1));
                }
            }
            catch (Exception e)
            {
                Log.Error("Cannot select blocks of " + characterId, e);
                result.Clear();
            }
            finally
            {
                cmd.Dispose();
                con.Close();
            }
            return result;
        }
        public static void InsertBlock(int characterId, string blockedName, string reason)
        {
            SqlConnection con = new SqlConnection(ConfigurationManager.ConnectionStrings["WorldCatalog"].ToString());
            SqlCommand cmd = new SqlCommand("INSERT INTO dbo.Blocks(CharacterID, BlockedName, Reason) VALUES(@CharID, @BlockedName, @Reason)", con);
            cmd.Parameters.Add("@CharID", System.Data.SqlDbType.Int).Value = characterId;
            cmd.Parameters.Add("@BlockedName", System.Data.SqlDbType.VarChar).Value = blockedName;
            cmd.Parameters.Add("@Reason", System.Data.SqlDbType.VarChar).Value = reason;
            // add command params here
            Log.Query(cmd);
            try
            {
                con.Open();
                cmd.ExecuteNonQuery();
            }
            catch (Exception e)
            {
                Log.Error("Cannot insert block relation " + characterId + "/" + blockedName, e);
            }
            finally
            {
                cmd.Dispose();
                con.Close();
            }
        }
        public static void RemoveBlock(int characterId, string blockedName)
        {
            SqlConnection con = new SqlConnection(ConfigurationManager.ConnectionStrings["WorldCatalog"].ToString());
            SqlCommand cmd = new SqlCommand("DELETE FROM dbo.Blocks WHERE CharacterID = @CharID AND BlockedName = @Name", con);
            cmd.Parameters.Add("@CharID", System.Data.SqlDbType.Int).Value = characterId;
            cmd.Parameters.Add("@Name", System.Data.SqlDbType.VarChar).Value = blockedName;
            Log.Query(cmd);
            try
            {
                con.Open();
                cmd.ExecuteNonQuery();
            }
            catch (Exception e)
            {
                Log.Error("Cannot remove block relation " + characterId + "/" + blockedName, e);
            }
            finally
            {
                cmd.Dispose();
                con.Close();
            }
        }
    }
}
