﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using System.Configuration;
using AionCxxLibrary.Utilities;

namespace WorldServer.DAO
{
    public class FriendsDAO
    {
        public static List<int> SelectFriends(int characterId)
        {
            SqlConnection con = new SqlConnection(ConfigurationManager.ConnectionStrings["WorldCatalog"].ToString());
            SqlCommand cmd = new SqlCommand("SELECT FriendID FROM dbo.Friends WHERE CharacterID = @CharID", con);
            cmd.Parameters.Add("@CharID", System.Data.SqlDbType.Int).Value = characterId;
            List<int> result = new List<int>();
            Log.Query(cmd);
            // add command params here
            try
            {
                con.Open();
                SqlDataReader rdr = cmd.ExecuteReader();
                while (rdr.Read())
                {
                    result.Add(rdr.GetInt32(0));
                }
            }
            catch (Exception e)
            {
                Log.Error("Cannot select friends of " + characterId, e);
                result.Clear();
            }
            finally
            {
                cmd.Dispose();
                con.Close();
            }
            return result;
        }
        public static void InsertFriend(int characterId, int friendId)
        {
            SqlConnection con = new SqlConnection(ConfigurationManager.ConnectionStrings["WorldCatalog"].ToString());
            SqlCommand cmd = new SqlCommand("INSERT INTO dbo.Friends(CharacterID, FriendID) VALUES(@CharID, @FriendID)", con);
            cmd.Parameters.Add("@CharID", System.Data.SqlDbType.Int).Value = characterId;
            cmd.Parameters.Add("@FriendID", System.Data.SqlDbType.Int).Value = friendId;
            Log.Query(cmd);
            // add command params here
            try
            {
                con.Open();
                cmd.ExecuteNonQuery();
            }
            catch (Exception e)
            {
                Log.Error("Cannot insert friend relation " + characterId + "/" + friendId, e);
            }
            finally
            {
                cmd.Dispose();
                con.Close();
            }
        }
        public static void RemoveFriend(int characterId, int friendId)
        {
            SqlConnection con = new SqlConnection(ConfigurationManager.ConnectionStrings["WorldCatalog"].ToString());
            SqlCommand cmd = new SqlCommand("DELETE FROM dbo.Friends WHERE CharacterID = @CharID AND FriendID = @FriendID)", con);
            cmd.Parameters.Add("@CharID", System.Data.SqlDbType.Int).Value = characterId;
            cmd.Parameters.Add("@FriendID", System.Data.SqlDbType.Int).Value = friendId;
            Log.Query(cmd);
            // add command params here
            try
            {
                con.Open();
                cmd.ExecuteNonQuery();
            }
            catch (Exception e)
            {
                Log.Error("Cannot remove friend relation " + characterId + "/" + friendId, e);
            }
            finally
            {
                cmd.Dispose();
                con.Close();
            }
        }
    }
}
