﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using AionCxxLibrary.Packets;
using AionCxxLibrary.Utilities;
using WorldServer.Services;
using WorldServer.Models;
using WorldServer.Helpers;
using WorldServer.Packets.Server;

namespace WorldServer.Packets.Client
{
    public class CPkt_ItemMove : AionClientPacket
    {
        public override void ProcessPacket()
        {
            int itemAionId = ReadInt();
            byte sourceStorage = ReadByte();
            byte destinationStorage = ReadByte();
            ushort destinationSlot = ReadUnsignedShort();
            ItemService.MoveItem(((WorldConnection)Connection).Character, itemAionId, sourceStorage, destinationStorage, (byte)destinationSlot);
        }
    }

    public class CPkt_ItemDelete : AionClientPacket
    {
        public override void ProcessPacket()
        {
            int ItemGUID = ReadInt();
            ItemService.DeleteItem(((WorldConnection)Connection).Character, ItemGUID);
        }
    }

    public class CPkt_ItemEquip : AionClientPacket
    {
        public override void ProcessPacket()
        {
            byte action = ReadByte();
            int slot = ReadInt();
            int guid = ReadInt();
            Log.Debug("EquipItem Action=" + action + " Slot=" + slot + " GUID=" + guid);
            switch (action)
            {
                case 0:
                    ItemService.EquipItem(((WorldConnection)Connection).Character, guid, slot);
                    break;
                case 1:
                    ItemService.UnequipItem(((WorldConnection)Connection).Character, guid);
                    break;
                case 2:
                    ItemService.SwitchWeapons(((WorldConnection)Connection).Character);
                    break;
            }
            SPkt_CharacterAppearance PktAppearance = new SPkt_CharacterAppearance((Connection as WorldConnection).Character);
            foreach (AionObject o in (Connection as WorldConnection).Character.NearbyObjects.FindAll(obj => obj is Character && MathHelper.GetDistance(obj, (Connection as WorldConnection).Character) <= 100))
            {
                (o as Character).SendPacket(PktAppearance);
            }
            Connection.SendPacket(PktAppearance);
        }
    }

}
