﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using AionCxxLibrary.Packets;
using WorldServer.Models;
using WorldServer.Services;
using AionCxxLibrary.Utilities;

namespace WorldServer.Packets.Server
{
    public class SPkt_SkillList : AionServerPacket
    {
        List<SkillEntry> Skills;
        int message;

        public SPkt_SkillList()
        {
            
        }

        public SPkt_SkillList(int skillId, short level, int message)
        {
            Skills = new List<SkillEntry>();
            Skills.Add(new SkillEntry() { SkillID = skillId, Level = level });
            this.message = message;
        }

        public override void WritePacket()
        {
            if(Skills == null)
                Skills = ((WorldConnection)Connection).Character.SkillList.ToList();
            WriteShort((short)Skills.Count); // Skill List Size
            if (Skills.Count > 0)
            {
                foreach (SkillEntry skill in Skills)
                {
                    WriteShort((short)skill.SkillID);
                    WriteShort(skill.Level);
                    WriteByte(0);
                    if (skill.SkillID == 30002 || skill.SkillID == 30003 || skill.SkillID == 40001 || skill.SkillID == 40002 || skill.SkillID == 40003 ||
                       skill.SkillID == 40004 || skill.SkillID == 40007 || skill.SkillID == 40008)
                        WriteByte((byte)(skill.Level / 100));
                    else
                        WriteByte(0);
                    WriteInt(0);
                    WriteByte(0); // TODO IsStigma
                }
            }

            WriteInt(message);

            if (message != 0)
            {
                WriteAionDescription(0); //TODO SkillNameId
                WriteString(Skills[0].Level.ToString());
            }
        }
    }

    public class SPkt_SkillInit : AionServerPacket
    {
        CastedSkill skill;
        public SPkt_SkillInit(CastedSkill skill)
        {
            this.skill = skill;
        }

        public override void WritePacket()
        {
            WriteInt(skill.Caster.GetGUID());
            WriteUnsignedShort(skill.SkTemplate.ID);
            WriteByte(skill.SkLevel);
            WriteByte(skill.TargetType);
            switch (skill.TargetType)
            {
                case 0:
                case 3:
                    WriteInt(skill.FirstTarget != null ? skill.FirstTarget.GetGUID() : 0);
                    break;
                case 1:
                    WriteFloat(skill.x);
                    WriteFloat(skill.y);
                    WriteFloat(skill.z);
                    break;
            }
            WriteUnsignedShort(skill.SkTemplate.CastingDelay);
            WriteInt(0);
        }
    }

    public class SPkt_SkillUse : AionServerPacket
    {
        CastedSkill skill;
        public SPkt_SkillUse(CastedSkill skill)
        {
            this.skill = skill;
        }
        public override void WritePacket()
        {
            WriteInt(skill.Caster.GetGUID());
            WriteByte(skill.TargetType);
            switch (skill.TargetType)
            {
                case 0:
                case 3:
                    WriteInt(skill.FirstTarget != null ? skill.FirstTarget.GetGUID() : 0);
                    break;
                case 1:
                    WriteFloat(skill.x);
                    WriteFloat(skill.y);
                    WriteFloat(skill.z);
                    break;
            }
            WriteUnsignedShort(skill.SkTemplate.ID);
            WriteByte(skill.SkLevel);
            WriteInt(skill.SkTemplate.DelayTime / 100); // cooldown
            WriteUnsignedShort(skill.ClientTime);
            WriteByte(0);

            WriteShort(0); //0 = classic damage, 32 = trigger next chain skill, 16 = no effect (dodged, parried ...)
            WriteByte(0); // Dash Param

            WriteShort(0); // Effect Count

        }
    }

}
