﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net.Sockets;
using System.Net;
using System.Threading;
using AionCxxLibrary.Network;
using AionCxxLibrary.Utilities;
using AionCxxLibrary.Packets;
using WorldServer.Packets;
using WorldServer.Packets.Server;
using WorldServer.Models;

namespace WorldServer.Services
{
    public class WorldConnection : AionConnection
    {
        public int[] Keys = new int[3];
        public Character Character;

        public WorldConnection(TcpClient client) : base(client, new WorldCrypter())
        {
            
        }

        public override void OnConnect()
        {
            base.OnConnect();
            Log.Debug("New connection from " + GetRemoteIP());
            BeginWrite(new SPkt_CryptKey());
        }

        public override void OnDisconnect()
        {
            base.OnDisconnect();
            if (Character != null)
            {
                CharacterService.OnDisconnect(Character);
                Character.Dispose();
            }
        }

    }    

    public class WorldListener : TcpListener
    {
        public WorldListener()
            : base(IPAddress.Parse("0.0.0.0"), 7777)
        {

        }
    }

    public class NetworkService
    {
        static WorldListener Listener;
        static List<int> ReconnectingAccounts = new List<int>();

        public static void Start()
        {
            AionCxxLibrary.Packets.Packets.instance = new WorldPackets();
            AionCxxLibrary.Packets.Packets.instance.Initialize();

            Listener = new WorldListener();
            Listener.Start();
            BeginAccept();
        }

        static WorldConnection CnxPreAlloc;

        static void BeginAccept()
        {
            CnxPreAlloc = new WorldConnection(null);
            Listener.BeginAcceptTcpClient(new AsyncCallback(EndAccept), Listener);
        }

        static void EndAccept(IAsyncResult ar)
        {
            TcpListener lsn = (TcpListener)ar.AsyncState;
            TcpClient client = lsn.EndAcceptTcpClient(ar);

            CnxPreAlloc.SetClient(client);
            CnxPreAlloc.OnConnect();

            BeginAccept();
        }

        public static void AddReconnectingAccount(int AccountID)
        {
            lock (ReconnectingAccounts)
            {
                if (ReconnectingAccounts.Contains(AccountID))
                    throw new Exception("This account is already set as Reconnecting !");
                ReconnectingAccounts.Add(AccountID);
            }
        }

        public static bool PollReconnectingAccount(int AccountID)
        {
            lock (ReconnectingAccounts)
            {
                if (ReconnectingAccounts.Contains(AccountID))
                {
                    ReconnectingAccounts.Remove(AccountID);
                    return true;
                }
                return false;
            }
        }

    }
}
