﻿/* Copyright (c) 2011-2012, Zetatron Consulting
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of Zetatron Consulting nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WorldServer.Models;
using AionCxxLibrary.Enums;
using AionCxxLibrary.Utilities;
using WorldServer.Helpers;
using WorldServer.Packets.Server;
using WorldServer.Commands;

namespace WorldServer.Services
{
    public class SocialService
    {
        public static void OnCharacterLogin(ref Character c)
        {
            c.Friends = new FriendsList(c);
            c.Blocks = new BlockList(c);
        }

        public static void BroadcastPublicMessage(Character c, PublicMessageType type, string message)
        {
            Log.Chat("(" + type.ToString() + ") " + c.Template.Name + ": " + message);

            if (message.StartsWith("."))
            {
                if (!c.isAdmin)
                    return;
                if (CxxCommands.Execute(c, message))
                    return;
            }

            SPkt_SocialPublicMessage pkt = new SPkt_SocialPublicMessage(type, c, message);

            switch (type)
            {
                case PublicMessageType.Regular:
                    // broadcast to players in 50m
                    foreach (AionObject obj in c.NearbyObjects.FindAll(o => o is Character && MathHelper.GetDistance(o, c) <= 50))
                        (obj as Character).SendPacket(pkt);
                    c.SendPacket(pkt);
                    break;
                case PublicMessageType.Shout:
                    // broadcast to players in 150m
                    foreach(AionObject obj in c.NearbyObjects.FindAll(o => o is Character && MathHelper.GetDistance(o, c) <= 150))
                        (obj as Character).SendPacket(pkt);
                    c.SendPacket(pkt);
                    break;
                default:
                    c.SendMessage("This chat type is not yet implemented.");
                    break;
            }
        }

    }
}
